from TDSUtilities import TDS
from PyTASTileType import *
from TDPUtilities import *
import os

##########################################
B = -1
N = 0
E = 1
S = 2
W = 3

##########################################
import svgwrite
from svgwrite import cm, mm

units = mm
#defaultTileColor = "white"#"#B0B0B0"
textStyle="font-family:Arial"
##########################################

# A sequence should be a dictionary containing:
#   name
#   domain list
#   domain list directions
#   domain list relative locations

def DrawSequenceToSVG(dwg, group, sequence):
    name        = sequence['name']
    domains     = sequence['domains']
    locations   = sequence['locations']
    directions  = sequence['directions']

    seqGroup = group.add(dwg.g(id=name))

    fontSize = 90
    spacerSize = fontSize

    xOffset = 0
    yOffset = 0
    for i in range(len(domains)):

        #domainGroup = seqGroup.add(dwg.g(id=name+":"+str(i)))
        x = xOffset

        if locations[i] == 'S':
            yOffset = yOffset + spacerSize

        if directions[i] == 180:
            xOffset = -xOffset

        xCoords = []
        for c in domains[i]:
            xCoords.append(str(x) + 'px')
            x = x + fontSize

        rot = directions[i]
        if rot == 180:
            xOffset = -xOffset

        text = dwg.text(domains[i],
                    #insert=(str(xOffset)+'px',0),
                    x=xCoords,
                    y=[0,0,0,0,0,0,0,0],
                    stroke='none',
                    #fill='#900',
                    font_size=str(fontSize)+'px',
                    #font_weight="bold",
                    font_family="Courier New",
                    transform="rotate("+str(rot)+", 0, 0)")
        xOffset = xOffset + len(domains[i])*fontSize + spacerSize

        seqGroup.add(text)

        #textLength = len(domains[i]) * font_size

        #text = dwg.text(domains[i],insert=(xOffset*units, yOffset*units),
        #                textLength=textSize*units,font_size=textSize*units,style=textStyle)
        #seqGroup.add(text)
        #xOffset = xOffset + textLength


def OuptutSequencesToSVG(svgFileName, sequenceList):
    if len(svgFileName) < 1:
        print("ERROR: In DrawSequencesToSVG but no output file name given.")
        return False
    if os.path.isabs(svgFileName) == False:
        svgFileName = "./" + svgFileName
        svgFileName = os.path.abspath(svgFileName)

    dwg = svgwrite.Drawing(filename=svgFileName, size=(2000,2000), debug=True)

    arrow_d = 'M -4,0 L 2,-3 L 1,0 L 2,3 L -4,0 z'
    arrow1 = dwg.marker(id='arrow1', orient='auto', overflow='visible')
    arrow1.add(dwg.path(d=arrow_d, fill='black', stroke='black',
                        transform='rotate(180) scale(0.7)'))
    dwg.defs.add(arrow1)
    arrow2 = dwg.marker(id='arrow2', orient='auto', overflow='visible')
    arrow2.add(dwg.path(d=arrow_d, fill='green', stroke='green',
                        transform='rotate(180) scale(0.7)'))
    dwg.defs.add(arrow2)

    for seq in sequenceList:
        DrawSequenceToSVG(dwg, dwg, seq)
    dwg.save()

sequenceList = [{'name':'tile1', 'domains':['ACTTTCGC','GCCAATCG','AACCTGAG','TCGTCGTC'], 'locations':[B,E,S,W], 'directions':[0,0,180,180]}]
OuptutSequencesToSVG('foo.svg', sequenceList)
quit()

def RotateDomains(domains, numRotations):
    domainList = domains.split(' ')
    newList = []
    for i in range(len(domainList)):
        newList.append(domainList[(i+numRotations)%len(domainList)])
    newDomains = ' '.join(newList)
    return newDomains

def ComplementAndReverseDomain(domain):
    comp = ""
    for base in domain:
        comp = comp + {
            'A' : 'T',
            'C' : 'G',
            'G' : 'C',
            'T' : 'A'}[base]
        
    return comp[::-1]

adapter_sequences_right_raw = """adpU1 CCCTTCTC TAGTCTTTAATGCGCG x y TCGTCTTTCCAGACGT
adpU2 GATGGCAATTCATCAA x y AACTGATAGCCCTAAA
adpU3 ATAGTGAATTTATCAA x y TATAATCCTGATTGTT
adpU4 GCCTGTTTATCAACAA x y AATCATAGGTCTGAGA
adpU5 TTACAGAGAGAATAAC x y TAGATAAGTCCTGAAC
adpU6 TGACGGAAATTATTCA x y ATAAAAACAGGGAAGC
adpU7 CTGAATTTACCGTTCC x y TTAAAGGTGAATTATC
adpU8 ACGATCTAAAGTTTTG x y AGTAAGCGTCATACAT"""

scaffold_left_loop_sequences_raw = """adLeft8 TGGTGCCTTCGTAGTG x y GCATTACGTATTTTAC
adLeft7 ACAATGATTAAAGTTG x y AAATTAAACCATCTCA
adLeft6 CTGATTTTTGATTTAT x y GGTCATTCTCGTTTTC
adLeft5 TGATGCCACCTTTTCA x y GCTCGCGCCCCAAATG
adLeft4 TTTACGATTACCGTTC x y ATCGATTCTCTTGTTT
adLeft3 GTTTCCGGCACCAGAA x y GCGGTGCCGGAAAGCT
adLeft2 ATGCAGCTGGCACGAC x y AGGTTTCCCGACTGGA
adLeft1 GGGGCTCCCTTTAGGG x y TTCCGATTTAGTGCTT"""

scaffold_left_loop_sequences_list = scaffold_left_loop_sequences_raw.split('\n')
adapter_sequences_left_list = []

llen = len(scaffold_left_loop_sequences_list)
for i in range(llen):
    scaffold_left_loop_sequence_list1 = scaffold_left_loop_sequences_list[i].split(' ')
    scaffold_left_loop_sequence_list2 = scaffold_left_loop_sequences_list[(i-1)%llen].split(' ')

    adapter_sequences_left_list.append([scaffold_left_loop_sequence_list1[0],
                                       ComplementAndReverseDomain(scaffold_left_loop_sequence_list1[4]),
                                       'x', 'y',
                                       ComplementAndReverseDomain(scaffold_left_loop_sequence_list2[1])])
    

tiles = \
[{'name': 'T00;_;_', 'parity': 1, 'E': 'glue_0',  'N': 'glue_1',  'S': 'glue_2',  'W': 'glue_3'},
 {'name': 'T01;_;_', 'parity': 0, 'E': 'glue_3',  'N': 'glue_2',  'S': 'glue_4',  'W': 'glue_5'},
 {'name': 'T02;_;_', 'parity': 1, 'E': 'glue_5',  'N': 'glue_4',  'S': 'glue_6',  'W': 'glue_7'},
 {'name': 'T03;_;_', 'parity': 0, 'E': 'glue_7',  'N': 'glue_6',  'S': 'glue_8',  'W': 'glue_9'},
 {'name': 'T04;_;_', 'parity': 1, 'E': 'glue_9',  'N': 'glue_8',  'S': 'glue_10', 'W': 'glue_11'},
 {'name': 'T05;_;_', 'parity': 0, 'E': 'glue_11', 'N': 'glue_10', 'S': 'glue_12', 'W': 'glue_13'},
 {'name': 'T06;_;_', 'parity': 1, 'E': 'glue_13', 'N': 'glue_12', 'S': 'glue_14', 'W': 'glue_15'},
 {'name': 'T07;_;_', 'parity': 0, 'E': 'glue_15', 'N': 'glue_14', 'S': 'glue_16', 'W': 'glue_17'},
 {'name': 'T08;_;_', 'parity': 1, 'E': 'glue_17', 'N': 'glue_16', 'S': 'glue_18', 'W': 'glue_19'},
 {'name': 'T09;_;_', 'parity': 0, 'E': 'glue_19', 'N': 'glue_18', 'S': 'glue_20', 'W': 'glue_21'},
 {'name': 'T10;_;_', 'parity': 1, 'E': 'glue_21', 'N': 'glue_20', 'S': 'glue_22', 'W': 'glue_23'}, # new tiles from here
 {'name': 'T11;_;_', 'parity': 0, 'E': 'glue_23', 'N': 'glue_22', 'S': 'glue_24', 'W': 'glue_25'},
 {'name': 'T12;_;_', 'parity': 1, 'E': 'glue_25', 'N': 'glue_24', 'S': 'glue_26', 'W': 'glue_27'},
 {'name': 'T13;_;_', 'parity': 0, 'E': 'glue_27', 'N': 'glue_26', 'S': 'glue_28', 'W': 'glue_29'},
 {'name': 'T14;_;_', 'parity': 1, 'E': 'glue_29', 'N': 'glue_28', 'S': 'glue_30', 'W': 'glue_31'},
 {'name': 'T15;_;_', 'parity': 0, 'E': 'glue_31', 'N': 'glue_30', 'S': 'glue_1',  'W': 'glue_0'},
]

tile_sequences_raw = """T00;_;_,ACCAAGAACT TTGTCAACAAT AAACAAATCCA ATCTTTCCGT,25nm,STD
T01;_;_,TTGTCTAGAGT TTGGGATGTT AGTTCTTGGT ATTGTTGACAA,25nm,STD
T02;_;_,TTATCCACGT TTCCTCCTATT ACTCTAGACAA AACATCCCAA,25nm,STD
T03;_;_,AAGGAAGTAGA TTCGAAAGGT ACGTGGATAA AATAGGAGGAA,25nm,STD
T04;_;_,AACCTCGAAT TACCAGATTCT TCTACTTCCTT ACCTTTCGAA,25nm,STD
T05;_;_,AGAATAGTCGT TTGTCAGTGT ATTCGAGGTT AGAATCTGGTA,25nm,STD
T06;_;_,ATCTGCTCAT TCTGATCTCTT ACGACTATTCT ACACTGACAA,25nm,STD
T07;_;_,AATGGATAGGT AGGTGTCTTT ATGAGCAGAT AAGAGATCAGA,25nm,STD
T08;_;_,TCAAGTTCCA TATCCTTAGCA ACCTATCCATT AAAGACACCT,25nm,STD
T09;_;_,AGTGATGATCT TTTAGGCTGT TGGAACTTGA TGCTAAGGATA,25nm,STD
T10;_;_,ACCCATTCAT TTCCTGATACT AGATCATCACT ACAGCCTAAA,25nm,STD
T11;_;_,TGCGTTAAAAT AGATGCGTAT ATGAATGGGT AGTATCAGGAA,25nm,STD
T12;_;_,AACCTTCACA ATCGTCTCATA ATTTTAACGCA ATACGCATCT,25nm,STD
T13;_;_,ATTCAGAGAGT TGGCATGATA TGTGAAGGTT TATGAGACGAT,25nm,STD
T14;_;_,TACCATGCTT TTGACCAATTT ACTCTCTGAAT TATCATGCCA,25nm,STD
T15;_;_,TGGATTTGTTT ACGGAAAGAT AAGCATGGTA AAATTGGTCAA,25nm,STD"""
tile_sequences_strings = tile_sequences_raw.split('\n')
if not len(tile_sequences_strings) == len(tiles):
    print("ERROR: Tile definitions and sequences lists are not of equal length!")
    quit()

conc = None
mix = None
tile_sequences = dict()
for tile_seq in tile_sequences_strings:
    tslist = tile_seq.split(',')
    tile_sequences[tslist[0]] = tslist[1]
    if len(tslist) > 2:
        conc = tslist[2]
    if len(tslist) > 3:
        mix = tslist[3]

tilesRot1 = []
tilesRot2 = []
tilesRot3 = []
for t in tiles:
    t['sequence'] = tile_sequences[t['name']]
    tileRot1 = t.copy()
    tileRot1['sequence'] = RotateDomains(t['sequence'], 1)
    tilesRot1.append(tileRot1)
    tileRot2 = t.copy()
    tileRot2['sequence'] = RotateDomains(t['sequence'], 2)
    tilesRot2.append(tileRot2)
    tileRot3 = t.copy()
    tileRot3['sequence'] = RotateDomains(t['sequence'], 3)
    tilesRot3.append(tileRot3)

extra = ""
if not conc is None:
    extra = "," + conc
if not mix is None:
    extra = extra + "," + mix
tileSequenceFile = open('16_helix_3_with_rotations.txt','w')
tileSequenceFile.write("Tiles rotated 0 times:\n")
counter = 1
for t in tiles:
    name = "tile" + str(counter) + "rot0"
    t['name'] = name
    tileSequenceFile.write(name + "," + t['sequence'] + extra + '\n')
    counter = counter + 1
tileSequenceFile.write("\nTiles rotated 1 time:\n")
counter = 1
for t in tilesRot1:
    name = "tile" + str(counter) + "rot1"
    t['name'] = name
    tileSequenceFile.write(name + "," + t['sequence'] + extra + '\n')
    counter = counter + 1
tileSequenceFile.write("\nTiles rotated 2 times:\n")
counter = 1
for t in tilesRot2:
    name = "tile" + str(counter) + "rot2"
    t['name'] = name
    tileSequenceFile.write(name + "," + t['sequence'] + extra + '\n')
    counter = counter + 1
tileSequenceFile.write("\nTiles rotated 3 times:\n")
counter = 1
for t in tilesRot3:
    name = "tile" + str(counter) + "rot3"
    t['name'] = name
    tileSequenceFile.write(name + "," + t['sequence'] + extra + '\n')
    counter = counter + 1
tileSequenceFile.close()


### Make the adapters

adapter_sequences_right_strings = adapter_sequences_right_raw.split('\n')
adapter_sequences_right_list = []
for adap_seq in adapter_sequences_right_strings:
    adapter_sequences_right_list.append(adap_seq.split(' '))

#for asr in adapter_sequences_right_list:
#    print(str(asr))

adapter_sequences_right_butt_first = dict() # 'butt_first' denotes that the tile attaches to the adapter butt first,
                                            # the adapter attaches to the scaffold head first
adapter_sequences_left_head_first = dict()

# Right side adapters
counter = 0
adapterctr = 1
for asr in adapter_sequences_right_list:
    asrButt = list(asr)
    #asrHead = list(asr)
    tile1 = tiles[counter]
    tile2 = tiles[(counter-2)%16]
    #print("tile1: " + str(tile1) + ", tile2: " + str(tile2))
    
    t1d1 = ComplementAndReverseDomain(tile1['sequence'].split(' ')[1])
    t2d2 = ComplementAndReverseDomain(tile2['sequence'].split(' ')[2])
    asrButt = [t1d1 if d == 'x' else d for d in asrButt[1:]]
    asrButt = [t2d2 if d == 'y' else d for d in asrButt[:]]
    adapter_sequences_right_butt_first['rightAdapter' + str(adapterctr)] = asrButt

##    t1d3 = ComplementDomain(tile1['sequence'].split(' ')[3])
##    t2d0 = ComplementDomain(tile2['sequence'].split(' ')[0])
##    asrHead = [t1d3 if d == 'x' else d for d in asrHead[1:]]
##    asrHead = [t2d0 if d == 'y' else d for d in asrHead[:]]
##    adapter_sequences_right_head_first['rightAdapter' + str(adapterctr)] = asrHead

    counter = counter + 2
    adapterctr = adapterctr + 1

rightAdaptersFile = open("16_helix_3_adapters_right.txt", 'w')
rightAdaptersFile.write("Right side adapters:\n")

rightAdapters = []
for asr in sorted(adapter_sequences_right_butt_first):
    seq = ' '.join(adapter_sequences_right_butt_first[asr])
    rightAdaptersFile.write(asr + "," + str(seq) + "\n")
    rightAdapters.append({'name':asr,'sequence':str(seq)})
rightAdaptersFile.close()

# Left side adapters
counter = 0
adapterctr = 1
for asr in adapter_sequences_left_list:
    asrList = list(asr)
    tile1 = tiles[counter]
    tile2 = tiles[(counter-2)%16]

    t1d1 = ComplementAndReverseDomain(tile1['sequence'].split(' ')[1])
    t2d2 = ComplementAndReverseDomain(tile2['sequence'].split(' ')[2])

    asrList = [t1d1 if d == 'x' else d for d in asrList[1:]]
    asrList = [t2d2 if d == 'y' else d for d in asrList[:]]

    adapter_sequences_left_head_first['leftAdapter' + str(adapterctr)] = asrList

    counter = counter + 2
    adapterctr = adapterctr + 1

leftAdaptersFile = open("16_helix_3_adapters_left.txt", 'w')
leftAdaptersFile.write("Left side adapters:\n")

leftAdapters = []
for asl in sorted(adapter_sequences_left_head_first):
    seq = ' '.join(adapter_sequences_left_head_first[asl])
    leftAdaptersFile.write(asl + "," + str(seq) + "\n")
    leftAdapters.append({'name':asl,'sequence':str(seq)})
leftAdaptersFile.close()

## Verify that tiles and adapters only have domains complementary to those that are expected

tileLists = [tiles, tilesRot1, tilesRot2, tilesRot3]
#tileLists = [[tiles[0]]]

matches = dict()

for adapters in [rightAdapters, leftAdapters]:
    for tl in tileLists:
        allTiles = adapters + tl

        for tile1 in allTiles:
            domains1 = tile1['sequence'].split(' ')
            tile1matches = []
            for tile2 in allTiles:
                if not tile1['name'] == tile2['name']:
                    #print(tile1['name'] + ":" + tile2['name'])
                    domains2 = tile2['sequence'].split(' ')
                    for d1 in domains1:
                        d1comp = ComplementAndReverseDomain(d1)
                        dc = 0
                        for d2 in domains2:
                            #print("     " + d1 + ":" + d2)
                            if d1comp == d2:
                                tile1matches.append(tile2['name'] + "-" + str(dc))
                            dc = dc + 1
            if len(tile1matches) > 0:
                if tile1['name'] in matches:
                    matches[tile1['name']] = matches[tile1['name']] + tile1matches
                else:
                    matches[tile1['name']] = tile1matches

print("Matches:")
for m in sorted(matches):
    print(m + ": " + str(matches[m]))

#print("Right side, head first adapters:")
#for asr in adapter_sequences_right_head_first:
#    print(str(asr))

tileSet = []
for tile in tiles:
        t = PyTASTileType(tile['name'])
        t.label = str(tile['parity'])
        t.sides[N].strength = 1
        t.sides[N].label = tile['N'].replace("glue_","")
        t.sides[E].strength = 1
        t.sides[E].label = tile['E'].replace("glue_","")
        t.sides[S].strength = 1
        t.sides[S].label = tile['S'].replace("glue_","")
        t.sides[W].strength = 1
        t.sides[W].label = tile['W'].replace("glue_","")
        tileSet.append(t)

tds = TDS('hard-coded-tiles.tds', tileSet)
tds.Write()

tdp = TDP('hard-coded-tiles.tdp', tdsFile = 'hard-coded-tiles.tds')
x = 0
y = 0
for i in range(3):
    for t in tileSet:
        if t.label == '1':
            tdp.AddTile(t.name, [x,y])
            x = x - 1
            y = y - 1
tdp.Write()

